@@ -1,6 +1,6 @@
 /*
 
-% Copyright (C) 2005-2013,2015 Alois Schloegl <alois.schloegl@ist.ac.at>
+% Copyright (C) 2005-2016 Alois Schloegl <alois.schloegl@ist.ac.at>
 % This file is part of the "BioSig for C/C++" repository
 % (biosig4c++) at http://biosig.sf.net/
 
@@ -40,7 +40,7 @@
 
 #define BIOSIG_VERSION_MAJOR 1
 #define BIOSIG_VERSION_MINOR 7
-#define BIOSIG_PATCHLEVEL 3
+#define BIOSIG_PATCHLEVEL 6
 // for backward compatibility
 #define BIOSIG_VERSION_STEPPING BIOSIG_PATCHLEVEL
 #define BIOSIG_VERSION (BIOSIG_VERSION_MAJOR * 10000 + BIOSIG_VERSION_MINOR * 100 + BIOSIG_PATCHLEVEL)
@@ -463,7 +463,7 @@ typedef struct HDR_STRUCT {
 				uint16_t physicalunits;
 				uint32_t value;
 			} *annotatedECG;
-	        } Section12;
+	        } Section12 ATT_DEPREC;	// deprecated as of Dec 2015 - will not be used in this way
 #endif
 	} SCP;
 #endif
