@@ -1,6 +1,6 @@
 /*
 
-    Copyright (C) 2010,2011,2012,2015 Alois Schloegl <alois.schloegl@ist.ac.at>
+    Copyright (C) 2010,2011,2012,2015,2016 Alois Schloegl <alois.schloegl@ist.ac.at>
 
     This file is part of the "BioSig for C/C++" repository
     (biosig4c++) at http://biosig.sf.net/
@@ -28,6 +28,29 @@
 #include <string.h>
 #include "../biosig-dev.h"
 
+uint16_t cfs_data_type(uint8_t dataType) {
+	switch (dataType) {
+	case 0:	//int8
+		return 1;
+	case 1: //uint8
+		return 3;
+	case 2: //int16
+		return 3;
+	case 3:	// uint16
+		return 4;
+	case 4: // int32
+		return 5;
+	case 5: //float
+		return 16;
+	case 6: // double
+		return 17;
+	case 7: // char
+		return 0;
+	default:
+		return 0xffff;
+	}
+}
+
 
 EXTERN_C void sopen_cfs_read(HDRTYPE* hdr) {
 /*
@@ -132,7 +155,10 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"%s(line %i) Channel #%i: %i<%s>/%i<%s>\n",_
 			//uint8_t  dataKind  = H2[43 + k*H2LEN];
 			//uint16_t byteSpace = leu16p(H2+44 + k*H2LEN);		// stride
 			//uint16_t next      = leu16p(H2+46 + k*H2LEN);
-			hc->GDFTYP = dataType < 5 ? dataType+1 : dataType+11;
+			hc->GDFTYP = cfs_data_type(dataType);
+			if (hc->GDFTYP > 580)	// sizeof GDFTYP_BITS[]
+				biosigERROR(hdr, B4C_DATATYPE_UNSUPPORTED, "CFS: unknown/unsupported data type !");
+
 			if (H2[43 + k * H2LEN] == 1) {
 				// Matrix data does not return an extra channel, but contains Markers and goes into the event table.
 				hc->OnOff = 0;
@@ -229,7 +255,7 @@ else if (VERBOSE_LEVEL>7)
 			}
 		}
 
-if (VERBOSE_LEVEL>7) fprintf(stdout,"\n******* Data Section variable information (n=%i,%i)*********\n", d,NumberOfDataSections);
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"\n******* Data Section variable information (n=%i,%i)*********\n", d,NumberOfDataSections);
 		datapos = LastDataSectionHeaderOffset; //H1LEN + H2LEN*hdr->NS + n*36;
 		// reverse order of data sections
 		uint32_t *DATAPOS = (uint32_t*)malloc(sizeof(uint32_t)*NumberOfDataSections);
@@ -238,7 +264,7 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"\n******* Data Section variable information
 		for (m = NumberOfDataSections; 0 < m; ) {
 			DATAPOS[--m] = datapos;
 			datapos      = leu32p(hdr->AS.Header + datapos);
-if (VERBOSE_LEVEL>7) fprintf(stdout, "%s:%i sopen_cfs_read started: section %"PRIi16" pos %"PRIuPTR" 0x%"PRIxPTR"\n",__FILE__,__LINE__,m,datapos,datapos);
+			if (VERBOSE_LEVEL>7) fprintf(stdout, "%s:%i sopen_cfs_read started: section %"PRIi16" pos %"PRIuPTR" 0x%"PRIxPTR"\n",__FILE__,__LINE__,m,datapos,datapos);
 		}
 
 		if (hdr->AS.SegSel[0] > NumberOfDataSections) {
@@ -300,15 +326,17 @@ if (VERBOSE_LEVEL>7) 		{
 
 			if (!leu32p(hdr->AS.Header+datapos+8)) continue; 	// empty segment
 
-if (VERBOSE_LEVEL>7) fprintf(stdout,"\n******* DATA SECTION --%03i-- %i *********\n",m,flag_ChanInfoChanged);
-if (VERBOSE_LEVEL>7) fprintf(stdout,"\n[DS#%3i] 0x%x 0x%x [0x%x 0x%x szChanData=%i] 0x02%x\n", m, FileHeaderSize, (int)datapos, leu32p(hdr->AS.Header+datapos), leu32p(hdr->AS.Header+datapos+4), leu32p(hdr->AS.Header+datapos+8), leu16p(hdr->AS.Header+datapos+12));
+			if (VERBOSE_LEVEL>7) fprintf(stdout,"\n******* DATA SECTION --%03i-- %i *********\n",m,flag_ChanInfoChanged);
+			if (VERBOSE_LEVEL>7) fprintf(stdout,"\n[DS#%3i] 0x%x 0x%x [0x%x 0x%x szChanData=%i] 0x02%x\n", m, FileHeaderSize, \
+						(int)datapos, leu32p(hdr->AS.Header+datapos), leu32p(hdr->AS.Header+datapos+4), \
+						leu32p(hdr->AS.Header+datapos+8), leu16p(hdr->AS.Header+datapos+12));
 
 			uint32_t sz    = 0;
 			uint32_t bpb   = 0, spr = 0;
 			hdr->AS.first  = 0;
 			hdr->AS.length = 0;
 			char flag_firstchan = 1;
-			uint32_t xspr0= 0;
+			uint32_t xspr0 = 0;
 
 			for (k = 0; k < hdr->NS; k++) {
 				uint8_t *pos = hdr->AS.Header + datapos + 30 + 24 * k;
@@ -321,26 +349,49 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"\n[DS#%3i] 0x%x 0x%x [0x%x 0x%x szChanData=
 				float Off    = lef32p(pos+12);
 				double XCal  = lef32p(pos+16);
 				double XOff  = lef32p(pos+20);// unused
-				if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): %i/%i %i/%i %f+%f change SPR:%i->%i, Cal:%f->%f, Off: %f->%f\n",__FILE__,__LINE__, (int)m, (int)NumberOfDataSections, (int)k,(int)hdr->NS, XCal, XOff, (int)hc->SPR,(int)xspr,hc->Cal,Cal,hc->Off,Off);
+				if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): %i/%i %i/%i %f+%f change SPR:%i->%i, Cal:%f->%f, Off: %f->%f\n", \
+							__FILE__,__LINE__, (int)m, (int)NumberOfDataSections, (int)k,(int)hdr->NS, XCal, XOff, \
+							(int)hc->SPR,(int)xspr,hc->Cal,Cal,hc->Off,Off);
 
-				if (k==0) xspr0 = xspr;
-				else if (xspr0 != xspr) {
-					if (VERBOSE_LEVEL>7) fprintf(stdout,"Error %s (line %i): %i/%i %i/%i change SPR:%i->%i\n",__FILE__,__LINE__, (int)m, (int)NumberOfDataSections, (int)k,(int)hdr->NS,(int)hc->SPR,(int)xspr);
-					biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: samples per record change - this is not supported yet.\n");
-				}
+				if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i) [%i %i] %i %i %p %i\n", __FILE__, __LINE__, m, k, xspr0, xspr, pos, datapos + 30 + 24 * k);
+
+				if (xspr0 == 0)
+					xspr0 = xspr;
+				else if (xspr>0)
+					xspr0 = lcm(xspr0,xspr);
+				else if (xspr==0)
+					if (VERBOSE_LEVEL>7) fprintf(stdout,"Warning : #%i of section %i contains %i samples \n", (int)k, (int)m, (int)xspr);
 
 				if (m > 0) {
 					if ( (hc->Cal != Cal)
 					  || (hc->Off != Off)
 					   )
-					biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: channel properties changes between segments - this is not supported yet.\n");
+					switch (hc->GDFTYP) {
+					case 0:
+					case 1:
+					case 2:
+					case 3:
+					case 4:
+					case 5:
+					case 6:
+					case 16:
+						hc->GDFTYP = 16; 	// float, single precision
+						break;
+					case 7:
+					case 8:
+					case 17:
+						hc->GDFTYP = 17; 	// double
+						break;
+					case 18:  	// quadruple
+					default: ;
+					}
 				}
 				else {
 					hc->Cal = Cal;
 					hc->Off = Off;
 				}
 
-if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x%g %g %g\n",m,k,spr,(int)SPR,hc->SPR,hc->Cal,hc->Off,hc->bi,xPhysDimScale[k], XCal, XOff);
+				if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x%g %g %g\n",m,k,spr,(int)SPR,hc->SPR,hc->Cal,hc->Off,hc->bi,xPhysDimScale[k], XCal, XOff);
 
 				double Fs = 1.0 / (xPhysDimScale[k] * XCal);
 				if ( (hc->OnOff == 0) || (XCal == 0.0) ) {
@@ -355,7 +406,7 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x
 				}
 
 				sz  += hc->SPR * GDFTYP_BITS[hc->GDFTYP] >> 3;
-				assert(hc->bi == bpb);
+				hc->bi = bpb;
 				bpb += GDFTYP_BITS[hc->GDFTYP]>>3;	// per single sample
 				hdr->AS.length += hc->SPR;
 
@@ -363,8 +414,7 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x
 
 			SPR += xspr0;
 			SZ  += sz;
-			if (hdr->AS.bpb != bpb)
-				biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: channel properties changes between segments - this is not supported yet.\n");
+			hdr->AS.bpb = bpb;
 		}
 
 		/*
@@ -391,7 +441,8 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x
 			hdr->AS.first  = 0;
 			hdr->AS.length = 0;
 			char flag_firstchan = 1;
-			uint32_t xspr0= 0;
+			uint32_t xspr0 = 0;
+			float Cal, Off;
 			for (k = 0; k < hdr->NS; k++) {
 				uint8_t *pos = hdr->AS.Header + datapos + 30 + 24 * k;
 
@@ -400,17 +451,21 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x
 				//uint32_t bi = leu32p(pos);	// unused
 				uint32_t xspr = leu32p(pos+4);
 				hc->SPR     = leu32p(pos+4);
-/*
 				hc->Cal     = lef32p(pos+8);
 				hc->Off     = lef32p(pos+12);
-*/
+				Cal         = lef32p(pos+8);
+				Off         = lef32p(pos+12);
 				double XCal = lef32p(pos+16);
 				double XOff = lef32p(pos+20);// unused
 
-				if (k==0) xspr0 = xspr;
-				assert(xspr0 == xspr);
+				if (xspr0 == 0)
+					xspr0 = xspr;
+				else if (xspr>0)
+					xspr0 = lcm(xspr0, xspr);
+				else if (xspr==0)
+					if (VERBOSE_LEVEL>7) fprintf(stdout,"Warning : #%i of section %i contains %i samples \n", (int)k, (int)m, (int)xspr);
 
-if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 409: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x%g %g %g %g %g\n",m,k,spr, (int)SPR, hc->SPR, hc->Cal, hc->Off, hc->bi, xPhysDimScale[k], lef32p(pos+8), lef32p(pos+12), XCal, XOff);
+				assert( (xspr==0) || ((xspr0 % xspr) == 0) );
 
 				double Fs = 1.0 / (xPhysDimScale[k] * XCal);
 				if ( (hc->OnOff == 0) || (XCal == 0.0) ) {
@@ -431,7 +486,7 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 409: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x
 
 			}
 
-			if (1) {
+			if (xspr0 > 0) {
 				/* hack: copy data into a single block (rawdata)
 				   this is used as a data cache, no I/O is needed in sread, at the cost that sopen is slower
 				   sread_raw does not attempt to reload the data
@@ -452,21 +507,23 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 409: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x
 					//uint8_t  dataKind  = H2[43 + k*H2LEN];		// equidistant, Subsidiary or Matrix data
 					//uint16_t stride = leu16p(H2+44 + k*H2LEN);		// byteSpace
 					uint16_t next      = leu16p(H2+46 + k*H2LEN);
-					uint16_t gdftyp    = dataType < 5 ? dataType+1 : dataType+11;
+					uint16_t gdftyp    = cfs_data_type(dataType);
+					if (gdftyp > 580)	// sizeof GDFTYP_BITS[]
+						biosigERROR(hdr, B4C_DATATYPE_UNSUPPORTED, "CFS: unknown/unsupported data type !");
 
 if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 412 #%i %i %i %i %i: %i @%p %i\n", k, hc->SPR, gdftyp,hc->GDFTYP, stride, memoffset, srcaddr, leu32p(hdr->AS.Header+datapos + 4) + leu32p(hdr->AS.Header + datapos + 30 + 24 * k));
-					if (gdftyp != hc->GDFTYP) {
-						biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: data type changes between segments - this is not supported yet.");
+
+					if (hc->OnOff==0) continue;	// Time and Marker channels are not supported, yet
+
+					if ((hc->SPR > 0) && (xspr0 != hc->SPR)) {
+						biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: samples-per-record (SPR) changes between channels - this is not supported yet.");
 					}
 
 					size_t k2;
-
-					for (k2 = 0; k2 < xspr0; k2++) {
+					if (gdftyp == hc->GDFTYP) {
+					    for (k2 = 0; k2 < xspr0; k2++) {
 
 						uint8_t *ptr = srcaddr + memoffset + k2*stride;
-
-if (VERBOSE_LEVEL>8) fprintf(stdout,"512 %i %i %i %i %i \n",(int)stride,(int)hc->bi,(int)SPR,(int)k2,(int)hdr->AS.bpb);
-
 						uint8_t *dest = hdr->AS.rawdata + hc->bi + (SPR + k2) * hdr->AS.bpb;
 						double val=NAN;
 
@@ -520,6 +577,78 @@ if (VERBOSE_LEVEL>8) fprintf(stdout,"512 %i %i %i %i %i \n",(int)stride,(int)hc-
 							hdr->EVENT.POS[hdr->EVENT.N] = lround( (val * hc->Cal + hc->Off) * hdr->SampleRate) + SPR;
 							hdr->EVENT.N++;
 						}
+					    }
+					}
+					else {
+					    for (k2 = 0; k2 < xspr0; k2++) {
+
+						uint8_t *ptr = srcaddr + memoffset + k2*stride;
+						uint8_t *dest = hdr->AS.rawdata + hc->bi + (SPR + k2) * hdr->AS.bpb;
+						double val;
+
+						if (hc->GDFTYP==16) {	// float
+							switch (gdftyp) {
+							// reorder for performance reasons - more frequent gdftyp's come first
+							case 3:  val = lei16p(ptr) * Cal + Off;
+								break;
+							case 4:  val = leu16p(ptr) * Cal + Off;
+								break;
+							case 16: val = lef32p(ptr) * Cal + Off;
+								break;
+							case 7:  val = lei64p(ptr) * Cal + Off;
+								break;
+							case 8:  val = leu64p(ptr) * Cal + Off;
+								break;
+							case 17: val = lef64p(ptr) * Cal + Off;
+								break;
+							case 0:  val = (*(   char*) ptr) * Cal + Off;
+								break;
+							case 1:  val = (*( int8_t*) ptr) * Cal + Off;
+								break;
+							case 2:  val = (*(uint8_t*) ptr) * Cal + Off;
+								break;
+							case 5:  val = lei32p(ptr) * Cal + Off;
+								break;
+							case 6:  val = leu32p(ptr) * Cal + Off;
+								break;
+							default:
+								val = NAN;
+								biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: invalid data type");
+							}
+
+							lef32a(val, dest);	// hdr->AS.rawdata uses the native endian format of the platform
+						}
+						else {
+							// any conversion to anything but float is not supported yet.
+							biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: data type changes between segments - this is not supported yet.");
+						}
+
+						// scaling has been already consider when converting to float, do not scale in sread anymore.
+						hc->Cal = 1.0;
+						hc->Off = 0.0;
+						if (hc->OnOff) continue;
+
+						if (!strncmp(hc->Label,"Marker",6) && hc->PhysDimCode==2176 && hc->GDFTYP==5 && next != 0) {
+							// matrix data might contain time markers.
+
+							// memory allocation for additional events - more efficient implementation would be nice
+							hdr->EVENT.TYP = (typeof(hdr->EVENT.TYP)) realloc(hdr->EVENT.TYP, (hdr->EVENT.N + NumberOfDataSections) * sizeof(*hdr->EVENT.TYP));
+							hdr->EVENT.POS = (typeof(hdr->EVENT.POS)) realloc(hdr->EVENT.POS, (hdr->EVENT.N + NumberOfDataSections) * sizeof(*hdr->EVENT.POS));
+
+							/*
+							char Desc[2]; Desc[0] = srcaddr[hdr->CHANNEL[next].bi + k2*stride]; Desc[1] = 0;
+								this does currently not work because FreeTextEvent expects that
+								the string constant is available as long as hdr, which is not the case here.
+							*/
+
+							// typically a single character within a 4 byte integer, this should be sufficient to ensure \0 termination
+							char *Desc = (char*)srcaddr + hdr->CHANNEL[next].bi + k2 * stride;
+							Desc[1] = 0;
+							FreeTextEvent(hdr, hdr->EVENT.N, Desc);
+							hdr->EVENT.POS[hdr->EVENT.N] = lround( (val * hc->Cal + hc->Off) * hdr->SampleRate) + SPR;
+							hdr->EVENT.N++;
+						}
+					    }
 					}
 					ns += hc->OnOff;
 				}
